@@ -89,6 +89,12 @@ PwizMsRunReader::initialize()
     {
       qDebug() << QString("Failed to read the data from file %1")
                     .arg(QString::fromStdString(file_name_std));
+
+      throw(PappsoException(
+        QString("Error reading file %1 in PwizMsRunReader, for msrun %2:\n%3")
+          .arg(mcsp_msRunId->getFileName())
+          .arg(mcsp_msRunId.get()->toString())
+          .arg(error.what())));
     }
 
   // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
@@ -134,7 +140,8 @@ PwizMsRunReader::initialize()
   if(msp_msData == nullptr)
     {
       throw(ExceptionNotPossible(
-        "Could not find a MSDataPtr matching the requested run id."));
+        QString("Could not find a MSDataPtr matching the requested run id : %1")
+          .arg(mcsp_msRunId.get()->toString())));
     }
 
 
@@ -211,6 +218,142 @@ PwizMsRunReader::getPwizSpectrumPtr(pwiz::msdata::SpectrumList *p_spectrum_list,
 }
 
 
+bool
+PwizMsRunReader::processRetentionTime(
+  pwiz::msdata::Spectrum *spectrum_p,
+  QualifiedMassSpectrum &qualified_mass_spectrum) const
+{
+
+  //  We now have to set the retention time at which this mass spectrum
+  //  was acquired. This is the scan start time.
+
+  if(!spectrum_p->scanList.scans[0].hasCVParam(
+       pwiz::msdata::MS_scan_start_time))
+    {
+      if(mcsp_msRunId.get()->getMzFormat() == MzFormat::MGF)
+        { // MGF could not have scan start time
+          qualified_mass_spectrum.setRtInSeconds(-1);
+        }
+      else
+        {
+          throw(ExceptionNotPossible(
+            "The spectrum has no scan start time value set."));
+        }
+    }
+  else
+    {
+      pwiz::data::CVParam retention_time_cv_param =
+        spectrum_p->scanList.scans[0].cvParam(pwiz::msdata::MS_scan_start_time);
+
+      // Try to get the units of the retention time value.
+
+      std::string unit_name = retention_time_cv_param.unitsName();
+      // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
+      //<< "Unit name for the retention time:"
+      //<< QString::fromStdString(unit_name);
+
+      if(unit_name == "second")
+        {
+          qualified_mass_spectrum.setRtInSeconds(
+            retention_time_cv_param.valueAs<double>());
+        }
+      else if(unit_name == "minute")
+        {
+          qualified_mass_spectrum.setRtInSeconds(
+            retention_time_cv_param.valueAs<double>() * 60);
+        }
+      else
+        throw(
+          ExceptionNotPossible("Could not determine the unit for the "
+                               "scan start time value."));
+    }
+
+  // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
+  //<< "Retention time for spectrum is:"
+  //<< qualified_mass_spectrum.getRtInSeconds();
+
+  // Old version not checking unit (by default unit is minutes for RT,
+  // not seconds)
+  //
+  // pappso_double retentionTime =
+  // QString(spectrum_p->scanList.scans[0]
+  //.cvParam(pwiz::msdata::MS_scan_start_time)
+  //.value.c_str())
+  //.toDouble();
+  // qualified_mass_spectrum.setRtInSeconds(retentionTime);
+
+  return true;
+}
+
+
+bool
+PwizMsRunReader::processDriftTime(
+  pwiz::msdata::Spectrum *spectrum_p,
+  QualifiedMassSpectrum &qualified_mass_spectrum) const
+{
+  // Not all the acquisitions have ion mobility data. We need to test
+  // that:
+
+  if(spectrum_p->scanList.scans[0].hasCVParam(
+       pwiz::msdata::MS_ion_mobility_drift_time))
+    {
+
+      // qDebug() << "as strings:"
+      //<< QString::fromStdString(
+      // spectrum_p->scanList.scans[0]
+      //.cvParam(pwiz::msdata::MS_ion_mobility_drift_time)
+      //.valueAs<std::string>());
+
+      pappso_double driftTime =
+        spectrum_p->scanList.scans[0]
+          .cvParam(pwiz::msdata::MS_ion_mobility_drift_time)
+          .valueAs<double>();
+
+      // qDebug() << "driftTime:" << driftTime;
+
+      // Old version requiring use of QString.
+      // pappso_double driftTime =
+      // QString(spectrum_p->scanList.scans[0]
+      //.cvParam(pwiz::msdata::MS_ion_mobility_drift_time)
+      //.value.c_str())
+      //.toDouble();
+
+      // Now make positively sure that the obtained value is correct.
+      // Note that I suffered a lot with Waters Synapt data that
+      // contained apparently correct drift time XML element that in
+      // fact contained either NaN or inf. When such mass spectra were
+      // encountered, the mz,i data were bogus and crashed the data
+      // loading functions. We just want to skip this kind of bogus mass
+      // spectrum by letting the caller  know that the drift time was
+      // bogus ("I" is Filippo Rusconi).
+
+      if(std::isnan(driftTime) || std::isinf(driftTime))
+        {
+          // qDebug() << "detected as nan or inf.";
+
+          return false;
+        }
+      else
+        {
+          // The mzML standard stipulates that drift times are in
+          // milliseconds.
+          qualified_mass_spectrum.setDtInMilliSeconds(driftTime);
+        }
+    }
+  // End of
+  // if(spectrum_p->scanList.scans[0].hasCVParam(
+  // pwiz::msdata::MS_ion_mobility_drift_time))
+  else
+    {
+      // Not a bogus mass spectrum but also not a drift spectrum, set -1
+      // as the drift time value.
+      qualified_mass_spectrum.setDtInMilliSeconds(-1);
+    }
+
+  return true;
+}
+
+
 QualifiedMassSpectrum
 PwizMsRunReader::qualifiedMassSpectrumFromPwizSpectrumPtr(
   const MassSpectrumId &massSpectrumId,
@@ -236,21 +379,24 @@ PwizMsRunReader::qualifiedMassSpectrumFromPwizSpectrumPtr(
 
       qualified_mass_spectrum.setMsLevel(msLevel);
 
-      //qDebug() << "for spectrum at index:" << massSpectrumId.getSpectrumIndex()
-               //<< "msLevel:" << msLevel
-               //<< "with number of precursors:" << spectrum_p->precursors.size();
-
       // We want to know if this spectrum is a fragmentation spectrum obtained
       // from a selected precursor ion.
 
-      if(spectrum_p->precursors.size() > 0)
+      std::size_t precursor_list_size = spectrum_p->precursors.size();
+
+      // qDebug() << "For spectrum at index:" <<
+      // massSpectrumId.getSpectrumIndex()
+      //<< "msLevel:" << msLevel
+      //<< "with number of precursors:" << precursor_list_size;
+
+      if(precursor_list_size > 0)
         {
-          //qDebug() << "The spectrum has precursor(s).";
 
           // Sanity check
-          if(msLevel <= 1)
+          if(msLevel < 2)
             {
-              qDebug() << "Going to throw.";
+              qDebug() << "Going to throw: msLevel cannot be less than two for "
+                          "a spectrum that has items in its Precursor list.";
 
               throw(ExceptionNotPossible(
                 "msLevel cannot be less than two for "
@@ -259,298 +405,182 @@ PwizMsRunReader::qualifiedMassSpectrumFromPwizSpectrumPtr(
 
           // See what is the first precursor in the list.
 
-          // qDebug() << "Now tring to get the precursor.";
-
-          pwiz::msdata::Precursor &precursor = spectrum_p->precursors.front();
-
-          // Set this variable ready as we need that default value in certain
-          // circumstances.
-
-          std::size_t precursor_spectrum_index =
-            std::numeric_limits<std::size_t>::max();
-
-          // qDebug() << "First precursor:" << precursor.spectrumID.c_str();
-
-          if(precursor.spectrumID.empty())
+          for(auto &precursor : spectrum_p->precursors)
             {
-              // qDebug() << "The precursor's spectrum ID is empty.";
 
-              if(mcsp_msRunId.get()->getMzFormat() == MzFormat::MGF)
+              // Set this variable ready as we need that default value in
+              // certain circumstances.
+
+              std::size_t precursor_spectrum_index =
+                std::numeric_limits<std::size_t>::max();
+
+              // The spectrum ID of the precursor might be empty.
+
+              if(precursor.spectrumID.empty())
                 {
-                  // qDebug()
-                  //<< "Format is MGF, precursor's spectrum ID can be empty.";
-                }
-              else
-                {
-                  // When performing Lumos Fusion fragmentation experiments in
-                  // Tune mode and with recording, the first spectrum of the
-                  // list is a fragmentation spectrum (ms level 2) that has no
-                  // identity for the precursor spectrum because there is no
-                  // full scan accquisition.
+                  // qDebug() << "The precursor's spectrum ID is empty.";
 
-#if 0
-                  // Let's try to understand if we are in this configuration.
-
-                  if(!massSpectrumId.getSpectrumIndex())
+                  if(mcsp_msRunId.get()->getMzFormat() == MzFormat::MGF)
                     {
-                      qDebug() << "This MS2 sprectrum is at index 0."
-                                  "This might indicate a fragmentation "
-                                  "experiment performed in Tune mode.";
-
-                      // The precursor spectrum index is initialized to max().
-                      // So, it is not going to be changed in the present
-                      // situation.
+                      // qDebug()
+                      //<< "Format is MGF, precursor's spectrum ID can be
+                      // empty.";
                     }
                   else
                     {
-                      qDebug() << "This MS2 spectrum has no precursor's "
-                                  "spectrum index.";
+                      // When performing Lumos Fusion fragmentation experiments
+                      // in Tune mode and with recording, the first spectrum of
+                      // the list is a fragmentation spectrum (ms level 2) that
+                      // has no identity for the precursor spectrum because
+                      // there is no full scan accquisition.
                     }
-                  // else
-                  //{
-                  // throw(
-                  // ExceptionNotPossible("Failed to get the id of the "
-                  //"precursor ion's spectrum."));
-                  //}
-#endif
                 }
+              // End of
+              // if(precursor.spectrumID.empty())
+              else
+                {
+                  // We could get a native precursor spectrum id, so convert
+                  // that native id to a spectrum index.
+
+                  qualified_mass_spectrum.setPrecursorNativeId(
+                    QString::fromStdString(precursor.spectrumID));
+
+                  if(qualified_mass_spectrum.getPrecursorNativeId().isEmpty())
+                    {
+                      // qDebug() << "The native id of the precursor spectrum is
+                      // empty.";
+                    }
+
+                  //  Get the spectrum index of the spectrum that contained the
+                  //  precursor ion.
+
+                  precursor_spectrum_index =
+                    msp_msData->run.spectrumListPtr->find(precursor.spectrumID);
+
+                  // Note that the Mascot MGF format has a peculiar handling of
+                  // the precursor ion stuff so we cannot throw.
+                  if(precursor_spectrum_index ==
+                     msp_msData->run.spectrumListPtr->size())
+                    {
+                      if(mcsp_msRunId.get()->getMzFormat() != MzFormat::MGF)
+                        {
+                          throw(ExceptionNotPossible(
+                            "Failed to find the index of the "
+                            "precursor ion's spectrum."));
+                        }
+                    }
+
+                  qualified_mass_spectrum.setPrecursorSpectrumIndex(
+                    precursor_spectrum_index);
+
+                  // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ <<
+                  // "()"
+                  //<< "Set the precursor spectrum index to:"
+                  //<< qualified_mass_spectrum.getPrecursorSpectrumIndex()
+                  //<< "for qualified mass spectrum:"
+                  //<< &qualified_mass_spectrum;
+                }
+
+              if(!precursor.selectedIons.size())
+                {
+                  qDebug()
+                    << "Going to throw The spectrum has msLevel > 1 but the "
+                       "precursor ions's selected ions list is empty..";
+
+                  throw(
+                    ExceptionNotPossible("The spectrum has msLevel > 1 but the "
+                                         "precursor ions's selected ions "
+                                         "list is empty."));
+                }
+
+              pwiz::msdata::SelectedIon &ion =
+                *(precursor.selectedIons.begin());
+
+              // selected ion m/z
+
+              pappso_double selected_ion_mz =
+                QString(
+                  ion.cvParam(pwiz::cv::MS_selected_ion_m_z).value.c_str())
+                  .toDouble();
+
+              // selected ion peak intensity
+
+              pappso_double selected_ion_peak_intensity =
+                QString(ion.cvParam(pwiz::cv::MS_peak_intensity).value.c_str())
+                  .toDouble();
+
+              // charge state
+
+              unsigned int selected_ion_charge_state =
+                QString(ion.cvParam(pwiz::cv::MS_charge_state).value.c_str())
+                  .toUInt();
+
+              // At this point we can craft a new PrecursorIonData instance and
+              // push it back to the vector.
+
+              PrecursorIonData precursor_ion_data(selected_ion_mz,
+                                                  selected_ion_charge_state,
+                                                  selected_ion_peak_intensity);
+
+              qualified_mass_spectrum.appendPrecursorIonData(
+                precursor_ion_data);
+
+              // General sum-up
+
+              // qDebug()
+              //<< "Appended new PrecursorIonData:"
+              //<< "mz:"
+              //<< qualified_mass_spectrum.getPrecursorIonData().back().mz
+              //<< "charge:"
+              //<< qualified_mass_spectrum.getPrecursorIonData().back().charge
+              //<< "intensity:"
+              //<< qualified_mass_spectrum.getPrecursorIonData()
+              //.back()
+              //.intensity;
             }
           // End of
-          // if(precursor.spectrumID.empty())
-          else
-            {
-              // We could get a native precursor spectrum id, so convert that
-              // native id to a spectrum index.
-
-              qualified_mass_spectrum.setPrecursorNativeId(
-                QString::fromStdString(precursor.spectrumID));
-
-              if(qualified_mass_spectrum.getPrecursorNativeId().isEmpty())
-                {
-                  // qDebug()
-                  //<< "The native id of the precursor spectrum is empty !!!";
-                }
-
-              //  Get the spectrum index of the spectrum that contained the
-              //  precursor ion.
-
-              precursor_spectrum_index =
-                msp_msData->run.spectrumListPtr->find(precursor.spectrumID);
-
-              // Note that the Mascot MGF format has a peculiar handling of the
-              // precursor ion stuff so we cannot throw.
-              if(precursor_spectrum_index ==
-                 msp_msData->run.spectrumListPtr->size())
-                {
-                  if(mcsp_msRunId.get()->getMzFormat() != MzFormat::MGF)
-                    {
-                      throw(
-                        ExceptionNotPossible("Failed to find the index of the "
-                                             "precursor ion's spectrum."));
-                    }
-                }
-
-              qualified_mass_spectrum.setPrecursorSpectrumIndex(
-                precursor_spectrum_index);
-
-              // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-              //<< "Set the precursor spectrum index to:"
-              //<< qualified_mass_spectrum.getPrecursorSpectrumIndex()
-              //<< "for qualified mass spectrum:"
-              //<< &qualified_mass_spectrum;
-            }
-
-          if(!precursor.selectedIons.size())
-            {
-              qDebug() << "Going to throw.";
-
-              throw(
-                ExceptionNotPossible("The spectrum has msLevel > 1 but the "
-                                     "precursor ions's selected ions "
-                                     "list is empty."));
-            }
-
-          pwiz::msdata::SelectedIon &ion = *(precursor.selectedIons.begin());
-
-          // selected ion m/z
-
-          pappso_double selected_ion_mz =
-            QString(ion.cvParam(pwiz::cv::MS_selected_ion_m_z).value.c_str())
-              .toDouble();
-
-          qualified_mass_spectrum.setPrecursorMz(selected_ion_mz);
-
-          // selected ion peak intensity
-
-          pappso_double selected_ion_peak_intensity =
-            QString(ion.cvParam(pwiz::cv::MS_peak_intensity).value.c_str())
-              .toDouble();
-
-          qualified_mass_spectrum.setPrecursorIntensity(
-            selected_ion_peak_intensity);
-
-          //  unsigned int test =
-          //  QString(ion.cvParam(pwiz::cv::MS_1200_series_LC_MSD_SL).value.c_str()).toUInt();
-          //  qDebug() << " test "<< test;
-
-          // charge state
-
-          unsigned int selected_ion_charge_state =
-            QString(ion.cvParam(pwiz::cv::MS_charge_state).value.c_str())
-              .toUInt();
-
-          if(selected_ion_charge_state > 0)
-            {
-              qualified_mass_spectrum.setPrecursorCharge(
-                selected_ion_charge_state);
-            }
-
-
-          // General sum-up
-          // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-          //<< "precursor_scan_number:"
-          //<< "selected_ion_m_z:" << selected_ion_mz
-          //<< "selected_ion_peak_intensity:"
-          //<< selected_ion_peak_intensity << "selected_ion_charge_state"
-          //<< selected_ion_charge_state;
+          // for(auto &precursor : spectrum_p->precursors)
         }
       // End of
-      // if(spectrum_p->precursors.size() > 0)
+      // if(precursor_list_size > 0)
       else
         {
           // Sanity check
-          
+
           // Unfortunately, logic here is defeated by some vendors that have
           // files with MS2 spectra without <precursorList>. Thus we have
           // spectrum_p->precursors.size() == 0 and msLevel > 1.
-#if 0 
-          if(msLevel != 1)
-            {
-              throw(
-                ExceptionNotPossible("msLevel cannot be different than 1 if "
-                                     "there is not a single precursor ion."));
-            }
-#endif 
+
+          // if(msLevel != 1)
+          //{
+          // throw(
+          // ExceptionNotPossible("msLevel cannot be different than 1 if "
+          //"there is not a single precursor ion."));
+          //}
         }
 
-      //  We now have to set the retention time at which this mass spectrum was
-      //  acquired. This is the scan start time.
+      // Sanity check.
 
-      if(!spectrum_p->scanList.scans[0].hasCVParam(
-           pwiz::msdata::MS_scan_start_time))
+      if(precursor_list_size !=
+         qualified_mass_spectrum.getPrecursorIonData().size())
         {
-          if(mcsp_msRunId.get()->getMzFormat() == MzFormat::MGF)
-            { // MGF could not have scan start time
-              qualified_mass_spectrum.setRtInSeconds(-1);
-            }
-          else
-            {
-              throw(ExceptionNotPossible(
-                "The spectrum has no scan start time value set."));
-            }
-        }
-      else
-        {
-          pwiz::data::CVParam retention_time_cv_param =
-            spectrum_p->scanList.scans[0].cvParam(
-              pwiz::msdata::MS_scan_start_time);
+          qDebug() << "Going to throw The number of precursors in the file is "
+                      "different from the number of precursors in memory.";
 
-          // Try to get the units of the retention time value.
-
-          std::string unit_name = retention_time_cv_param.unitsName();
-          // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-          //<< "Unit name for the retention time:"
-          //<< QString::fromStdString(unit_name);
-
-          if(unit_name == "second")
-            {
-              qualified_mass_spectrum.setRtInSeconds(
-                retention_time_cv_param.valueAs<double>());
-            }
-          else if(unit_name == "minute")
-            {
-              qualified_mass_spectrum.setRtInSeconds(
-                retention_time_cv_param.valueAs<double>() * 60);
-            }
-          else
-            throw(ExceptionNotPossible(
-              "Could not determine the unit for the scan start time value."));
+          throw pappso::PappsoException(
+            QObject::tr("The number of precursors in the file is different "
+                        "from the number of precursors in memory."));
         }
 
-      // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-      //<< "Retention time for spectrum is:"
-      //<< qualified_mass_spectrum.getRtInSeconds();
+      // if(precursor_list_size == 1)
+      //{
+      // qDebug() << "Trying to get the mz value of the unique precursor ion:"
+      //<< qualified_mass_spectrum.getPrecursorMz();
+      //}
 
-      // Old version not checking unit (by default unit is minutes for RT,
-      // not seconds)
-      //
-      // pappso_double retentionTime =
-      // QString(spectrum_p->scanList.scans[0]
-      //.cvParam(pwiz::msdata::MS_scan_start_time)
-      //.value.c_str())
-      //.toDouble();
-      // qualified_mass_spectrum.setRtInSeconds(retentionTime);
+      processRetentionTime(spectrum_p, qualified_mass_spectrum);
 
-      // Not all the acquisitions have ion mobility data. We need to test
-      // that:
-
-      if(spectrum_p->scanList.scans[0].hasCVParam(
-           pwiz::msdata::MS_ion_mobility_drift_time))
-        {
-
-          // qDebug() << "as strings:"
-          //<< QString::fromStdString(
-          // spectrum_p->scanList.scans[0]
-          //.cvParam(pwiz::msdata::MS_ion_mobility_drift_time)
-          //.valueAs<std::string>());
-
-          pappso_double driftTime =
-            spectrum_p->scanList.scans[0]
-              .cvParam(pwiz::msdata::MS_ion_mobility_drift_time)
-              .valueAs<double>();
-
-          // qDebug() << "driftTime:" << driftTime;
-
-          // Old version requiring use of QString.
-          // pappso_double driftTime =
-          // QString(spectrum_p->scanList.scans[0]
-          //.cvParam(pwiz::msdata::MS_ion_mobility_drift_time)
-          //.value.c_str())
-          //.toDouble();
-
-          // Now make positively sure that the obtained value is correct.
-          // Note that I suffered a lot with Waters Synapt data that
-          // contained apparently correct drift time XML element that in
-          // fact contained either NaN or inf. When such mass spectra were
-          // encountered, the mz,i data were bogus and crashed the data loading
-          // functions. We just want to skip this kind of bogus mass spectrum by
-          // letting the caller  know that the drift time was bogus ("I" is
-          // Filippo Rusconi).
-
-          if(std::isnan(driftTime) || std::isinf(driftTime))
-            {
-              // qDebug() << "detected as nan or inf.";
-
-              ok = false;
-
-              return qualified_mass_spectrum;
-            }
-          else
-            {
-              // The mzML standard stipulates that drift times are in
-              // milliseconds.
-              qualified_mass_spectrum.setDtInMilliSeconds(driftTime);
-            }
-        }
-      // End of
-      // if(spectrum_p->scanList.scans[0].hasCVParam(
-      // pwiz::msdata::MS_ion_mobility_drift_time))
-      else
-        {
-          // Not a bogus mass spectrum but also not a drift spectrum, set -1 as
-          // the drift time value.
-          qualified_mass_spectrum.setDtInMilliSeconds(-1);
-        }
+      processDriftTime(spectrum_p, qualified_mass_spectrum);
 
       // for(pwiz::data::CVParam cv_param : ion.cvParams)
       //{
@@ -601,8 +631,8 @@ PwizMsRunReader::qualifiedMassSpectrumFromPwizSpectrumPtr(
           // lc = localeconv ();
           // qDebug() << " env=" << localeconv () << " lc->decimal_point "
           // << lc->decimal_point;
-          //  qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "() "<<
-          //  spectrum.size();
+          //  qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()
+          //  "<< spectrum.size();
           MassSpectrumSPtr spectrum_sp = spectrum.makeMassSpectrumSPtr();
           qualified_mass_spectrum.setMassSpectrumSPtr(spectrum_sp);
 
@@ -637,6 +667,9 @@ PwizMsRunReader::qualifiedMassSpectrumFromPwizSpectrumPtr(
   // setlocale(LC_ALL, env.c_str());
 
   ok = true;
+
+  // qDebug() << "QualifiedMassSpectrum: " <<
+  // qualified_mass_spectrum.toString();
   return qualified_mass_spectrum;
 }
 
@@ -755,6 +788,7 @@ PwizMsRunReader::readSpectrumCollection(
   SpectrumCollectionHandlerInterface &handler)
 {
 
+  acquireDevice();
   // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()";
 
   // We want to iterate in the pwiz-spectrum-list and for each pwiz-spectrum
@@ -794,8 +828,8 @@ PwizMsRunReader::readSpectrumCollection(
   for(std::size_t iter = 0; iter < spectrum_list_size; iter++)
     {
 
-      // If the user of this reader instance wants to stop reading the spectra,
-      // then break this loop.
+      // If the user of this reader instance wants to stop reading the
+      // spectra, then break this loop.
       if(handler.shouldStop())
         {
           qDebug() << "The operation was cancelled. Breaking the loop.";
@@ -814,7 +848,8 @@ PwizMsRunReader::readSpectrumCollection(
 
        * if(!native_pwiz_spectrum_sp->hasBinaryData())
               {
-                // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "
+                // qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ <<
+       "
        ()"
                 //<< "native pwiz spectrum is empty, continuing.";
                 continue;
@@ -850,8 +885,8 @@ PwizMsRunReader::readSpectrumCollection(
           continue;
         }
 
-      // Before handing the mass spectrum out to the handler, see if the native
-      // mass spectrum was empty or not.
+      // Before handing the mass spectrum out to the handler, see if the
+      // native mass spectrum was empty or not.
 
       // if(!native_pwiz_spectrum_sp->defaultArrayLength)
       // qDebug() << "The mass spectrum has not defaultArrayLength";
@@ -889,5 +924,21 @@ PwizMsRunReader::hasScanNumbers() const
   return m_hasScanNumbers;
 }
 
+bool
+PwizMsRunReader::releaseDevice()
+{
+  msp_msData = nullptr;
+  return true;
+}
+
+bool
+PwizMsRunReader::acquireDevice()
+{
+  if(msp_msData == nullptr)
+    {
+      initialize();
+    }
+  return true;
+}
 
 } // namespace pappso
