@@ -61,6 +61,11 @@ class TimsMsRunReader : public MsRunReader
 
   virtual bool hasScanNumbers() const override;
 
+  virtual bool releaseDevice() override;
+
+  virtual bool acquireDevice() override;
+
+
   protected:
   virtual void initialize() override;
   virtual bool accept(const QString &file_name) const override;
