@@ -269,5 +269,17 @@ XyMsRunReader::spectrumListSize() const
   return 1;
 }
 
+bool
+XyMsRunReader::releaseDevice()
+{
+  return true;
+}
+
+bool
+XyMsRunReader::acquireDevice()
+{
+  return true;
+}
+
 
 } // namespace pappso
