@@ -29,6 +29,11 @@ class XyMsRunReader : public MsRunReader
 
   virtual std::size_t spectrumListSize() const override;
 
+  virtual bool releaseDevice() override;
+
+  virtual bool acquireDevice() override;
+
+
   protected:
   QString m_fileName;
   virtual void initialize() override;
