@@ -151,8 +151,8 @@ PrecisionFactory::getPpmInstance(pappso_double value)
   if(!value)
     throw ExceptionNotPossible(
       QObject::tr("Fatal error at precision.cpp "
-				"-- ERROR trying to set a Resolution precision value of 0. "
-				"Program aborted."));
+                  "-- ERROR trying to set a Resolution precision value of 0. "
+                  "Program aborted."));
 
   MapPpmPrecision::iterator it = m_mapPpm.find(value);
 
@@ -178,8 +178,8 @@ PrecisionFactory::getResInstance(pappso_double value)
   if(!value)
     throw ExceptionNotPossible(
       QObject::tr("Fatal error at precision.cpp "
-				"-- ERROR trying to set a Resolution precision value of 0. "
-				"Program aborted."));
+                  "-- ERROR trying to set a Resolution precision value of 0. "
+                  "Program aborted."));
 
   MapResPrecision::iterator it = m_mapRes.find(value);
 
@@ -198,6 +198,41 @@ PrecisionFactory::getResInstance(pappso_double value)
   return it->second;
 }
 
+PrecisionPtr
+PrecisionFactory::getPrecisionPtrFractionInstance(PrecisionPtr origin,
+                                                  double fraction)
+{
+  PrecisionUnit unit = origin->unit();
+  double value       = origin->getNominal() * fraction;
+
+
+  return getPrecisionPtrInstance(unit, value);
+}
+
+PrecisionPtr
+PrecisionFactory::getPrecisionPtrInstance(PrecisionUnit unit, double value)
+{
+
+  switch(unit)
+    {
+      case PrecisionUnit::dalton:
+        return getDaltonInstance(value);
+      case PrecisionUnit::ppm:
+        return getPpmInstance(value);
+      case PrecisionUnit::res:
+        return getResInstance(value);
+      case PrecisionUnit::mz:
+        return getDaltonInstance(value);
+      case PrecisionUnit::none:
+        throw ExceptionNotPossible(QObject::tr("Unknown precision unit"));
+
+      default:
+        throw ExceptionNotPossible(QObject::tr("Unknown precision unit"));
+        break;
+    }
+
+  return nullptr;
+}
 
 DaltonPrecision::DaltonPrecision(pappso_double x) : PrecisionBase(x)
 {
