@@ -217,11 +217,12 @@ FilterGreatestYperWindow::FilterGreatestYperWindow(
   qDebug();
 }
 
-
 FilterGreatestYperWindow &
 FilterGreatestYperWindow::operator=(const FilterGreatestYperWindow &other)
 {
+  qDebug();
   m_numberOfPoints = other.m_numberOfPoints;
+  m_xWindowRange   = other.m_xWindowRange;
 
   return *this;
 }
@@ -231,12 +232,14 @@ Trace &
 FilterGreatestYperWindow::filter(Trace &data_points) const
 {
 
-  Trace new_trace;
+  std::vector<DataPoint> new_trace(data_points);
+  data_points.clear();
 
   int window_number                     = 0;
   int old_window_number                 = -1;
   std::size_t number_of_peaks_in_window = 0;
-  auto itbegin                          = new_trace.begin();
+  auto itbegin                          = data_points.begin();
+  std::vector<DataPoint>::iterator it_min;
 
 
   // std::sort(data_points.begin(),
@@ -246,7 +249,7 @@ FilterGreatestYperWindow::filter(Trace &data_points) const
 
   qDebug() << " m_xWindowRange=" << m_xWindowRange
            << " m_numberOfPoints=" << m_numberOfPoints;
-  for(auto &data_point : data_points)
+  for(const pappso::DataPoint &data_point : new_trace)
     {
       qDebug() << " data_point.x=" << data_point.x
                << " data_point.y=" << data_point.y;
@@ -256,40 +259,44 @@ FilterGreatestYperWindow::filter(Trace &data_points) const
         {
           old_window_number         = window_number;
           number_of_peaks_in_window = 0;
-          itbegin                   = new_trace.end();
+          itbegin                   = data_points.end();
         }
       if(number_of_peaks_in_window < m_numberOfPoints)
         {
           qDebug();
-          new_trace.push_back(data_point);
+          data_points.push_back(data_point);
           number_of_peaks_in_window++;
           if(number_of_peaks_in_window == 1)
             {
-              itbegin = new_trace.begin() + (new_trace.size() - 1);
+              itbegin = data_points.begin() + (data_points.size() - 1);
             }
         }
       else
         {
           qDebug();
-          auto it_min = minYDataPoint(itbegin, new_trace.end());
-          qDebug();
-          if(it_min != new_trace.end())
+
+          it_min = minYDataPoint(itbegin, data_points.end());
+          if(it_min != data_points.end())
             {
               qDebug();
               if(it_min->y < data_point.y)
                 {
                   qDebug();
-                  it_min->x = data_point.x;
-                  it_min->y = data_point.y;
+                  *it_min = data_point;
+                  // it_min->x = data_point.x;
+                  // it_min->y = data_point.y;
                 }
             }
         }
     }
   qDebug();
-  new_trace.sortX();
+  // new_trace.sortX();
   // qDebug() << new_trace.size();
-  data_points = std::move(new_trace);
+  // data_points.clear();
+  // data_points = new_trace;
+  // data_points = std::move(new_trace);
   // qDebug() << data_points.size();
+  data_points.sortX();
   qDebug();
   return data_points;
 }
@@ -402,8 +409,8 @@ MassSpectrumFilterGreatestItensities::MassSpectrumFilterGreatestItensities(
 }
 
 MassSpectrumFilterGreatestItensities &
-MassSpectrumFilterGreatestItensities::
-operator=(const MassSpectrumFilterGreatestItensities &other)
+MassSpectrumFilterGreatestItensities::operator=(
+  const MassSpectrumFilterGreatestItensities &other)
 {
   m_filterGreatestY = other.m_filterGreatestY;
 
