@@ -97,8 +97,8 @@ MapTrace::initialize(const std::vector<pappso_double> &xVector,
   if(xVector.size() != yVector.size())
     throw ExceptionNotPossible(
       QObject::tr("Fatal error at msrundatasettreenode.cpp "
-				"-- ERROR xVector and yVector must have the same size."
-				"Program aborted."));
+                  "-- ERROR xVector and yVector must have the same size."
+                  "Program aborted."));
 
   for(std::size_t iter = 0; iter < xVector.size(); ++iter)
     {
@@ -164,7 +164,7 @@ MapTrace::makeMapTraceCstSPtr() const
 
 
 std::vector<pappso_double>
-MapTrace::firstToVector() const
+MapTrace::xValues()
 {
   std::vector<pappso_double> vector;
 
@@ -176,7 +176,7 @@ MapTrace::firstToVector() const
 
 
 std::vector<pappso_double>
-MapTrace::secondToVector() const
+MapTrace::yValues()
 {
   std::vector<pappso_double> vector;
 
@@ -187,20 +187,6 @@ MapTrace::secondToVector() const
 }
 
 
-std::vector<pappso_double>
-MapTrace::xValues()
-{
-  return firstToVector();
-}
-
-
-std::vector<pappso_double>
-MapTrace::yValues()
-{
-  return secondToVector();
-}
-
-
 Trace
 MapTrace::toTrace() const
 {
