@@ -4,6 +4,7 @@
 #include <map>
 #include <cmath>
 #include <algorithm>
+#include <iostream>
 
 #include <QDebug>
 
@@ -82,7 +83,6 @@ findFirstGreaterX(std::vector<DataPoint>::const_iterator begin,
   });
 }
 
-
 std::vector<DataPoint>::iterator
 findDifferentYvalue(std::vector<DataPoint>::iterator begin,
                     std::vector<DataPoint>::iterator end,
@@ -156,6 +156,9 @@ maxYDataPoint(std::vector<DataPoint>::iterator begin,
 }
 
 
+// As long as next DataPoint has its y value less or equal to prev's,
+// move along down the container. That is, continue moving is
+// direction is downhill to the end of the container (its back).
 std::vector<DataPoint>::const_iterator
 moveLowerYRigthDataPoint(const Trace &trace,
                          std::vector<DataPoint>::const_iterator begin)
@@ -164,6 +167,8 @@ moveLowerYRigthDataPoint(const Trace &trace,
     return begin;
   auto it     = begin + 1;
   auto result = begin;
+  // Move along as long as next point's y value is less
+  // or equal to prev point's y value (FR, check).
   while((it != trace.end()) && (it->y <= result->y))
     {
       it++;
@@ -180,6 +185,10 @@ moveLowerYLeftDataPoint(const Trace &trace,
     return begin;
   auto it     = begin - 1;
   auto result = begin;
+
+  // As long as prev datapoint has y value less or equal to next,
+  // move along up the container. That is, continue moving if
+  // direction is downhill to the beginning of the container (its front).
   while((it != trace.begin()) && (it->y <= result->y))
     {
       it--;
@@ -368,6 +377,13 @@ Trace::Trace()
 }
 
 
+Trace::Trace(const std::vector<pappso_double> &xVector,
+             const std::vector<pappso_double> &yVector)
+{
+  initialize(xVector, yVector);
+}
+
+
 Trace::Trace(
   const std::vector<std::pair<pappso_double, pappso_double>> &dataPoints)
 {
@@ -378,15 +394,20 @@ Trace::Trace(
       push_back(DataPoint(dataPoint));
     }
 
-  std::sort(begin(), end(), [](const DataPoint &a, const DataPoint &b) {
-    return (a.y < b.y);
-  });
+  sortX();
+  // std::sort(begin(), end(), [](const DataPoint &a, const DataPoint &b) {
+  // return (a.x < b.x);
+  //});
 }
 
 
 Trace::Trace(const std::vector<DataPoint> &dataPoints)
   : std::vector<DataPoint>(dataPoints)
 {
+  sortX();
+  // std::sort(begin(), end(), [](const DataPoint &a, const DataPoint &b) {
+  // return (a.x < b.x);
+  //});
 }
 
 
@@ -394,6 +415,11 @@ Trace::Trace(const std::vector<DataPoint> &&dataPoints)
   : std::vector<DataPoint>(std::move(dataPoints))
 {
   // This constructor used by the MassSpectrum && constructor.
+
+  sortX();
+  // std::sort(begin(), end(), [](const DataPoint &a, const DataPoint &b) {
+  // return (a.x < b.x);
+  //});
 }
 
 
@@ -401,6 +427,8 @@ Trace::Trace(const MapTrace &map_trace)
 {
   for(auto &&item : map_trace)
     push_back(DataPoint(item.first, item.second));
+
+  // No need to sort, maps are sorted by key (that is, x).
 }
 
 Trace::Trace(const Trace &other) : std::vector<DataPoint>(other)
@@ -430,16 +458,26 @@ Trace::initialize(const std::vector<pappso_double> &xVector,
     throw ExceptionNotPossible(
       "trace.cpp -- ERROR xVector and yVector must have the same size.");
 
-  // Do not force the release of the all the vector space, because we prefer
-  // resizing.  clear();
-
-  resize(xVector.size());
+  // We are initializing, not appending.
+  erase(begin(), end());
 
   for(std::size_t iter = 0; iter < xVector.size(); ++iter)
     {
       push_back(DataPoint(xVector.at(iter), yVector.at(iter)));
     }
 
+  sortX();
+  // std::sort(begin(), end(), [](const DataPoint &a, const DataPoint &b) {
+  // return (a.x < b.x);
+  //});
+
+#if 0
+  for(auto &item : *this)
+  {
+    std::cout << item.x << "-" << item.y;
+  }
+#endif
+
   return size();
 }
 
@@ -448,16 +486,16 @@ size_t
 Trace::initialize(const std::map<pappso_double, pappso_double> &map)
 {
 
-  // Do not force the release of the all the vector space, because we prefer
-  // resizing.  clear(false);
-
-  resize(map.size());
+  // We are initializing, not appending.
+  erase(begin(), end());
 
   for(auto &&item : map)
     {
       push_back(DataPoint(item.first, item.second));
     }
 
+  // No need to sort, maps are sorted by key (that is, x).
+
   return size();
 }
 
@@ -503,26 +541,30 @@ Trace::makeTraceCstSPtr() const
 
 
 std::vector<pappso_double>
-Trace::xToVector() const
+Trace::xValues() const
 {
-  std::vector<pappso_double> vector;
+  std::vector<pappso_double> values;
 
   for(auto &&dataPoint : *this)
-    vector.push_back(dataPoint.x);
+    {
+      values.push_back(dataPoint.x);
+    }
 
-  return vector;
+  return values;
 }
 
 
 std::vector<pappso_double>
-Trace::yToVector() const
+Trace::yValues() const
 {
-  std::vector<pappso_double> vector;
+  std::vector<pappso_double> values;
 
   for(auto &&dataPoint : *this)
-    vector.push_back(dataPoint.y);
+    {
+      values.push_back(dataPoint.y);
+    }
 
-  return vector;
+  return values;
 }
 
 
@@ -574,7 +616,7 @@ Trace::toMap() const
 
 
 std::vector<DataPoint>::iterator
-Trace::dataPointIteratorxWithX(pappso_double value)
+Trace::dataPointIteratorWithX(pappso_double value)
 {
   auto iterator =
     std::find_if(begin(), end(), [value](const DataPoint &dataPoint) {
@@ -586,7 +628,7 @@ Trace::dataPointIteratorxWithX(pappso_double value)
 
 
 std::vector<DataPoint>::const_iterator
-Trace::dataPointCstIteratorxWithX(pappso_double value) const
+Trace::dataPointCstIteratorWithX(pappso_double value) const
 {
   auto iterator =
     std::find_if(begin(), end(), [value](const DataPoint &dataPoint) {
@@ -601,7 +643,7 @@ std::size_t
 Trace::dataPointIndexWithX(pappso_double value) const
 {
   std::vector<DataPoint>::const_iterator iterator =
-    dataPointCstIteratorxWithX(value);
+    dataPointCstIteratorWithX(value);
 
   if(iterator != end())
     return std::distance(begin(), iterator);
@@ -648,7 +690,7 @@ Trace::minYDataPoint() const
 {
   auto dataPoint = std::min_element(
     begin(), end(), [](const DataPoint &a, const DataPoint &b) {
-      return (b.y < a.y);
+      return (a.y < b.y);
     });
 
   if(dataPoint == end())
@@ -721,8 +763,10 @@ pappso_double
 Trace::sumY(double mzStart, double mzEnd) const
 {
   auto begin_it = findFirstEqualOrGreaterX(this->begin(), this->end(), mzStart);
+  auto end_it = findFirstGreaterX(begin_it, this->end(), mzEnd) ;
+
   return sumYTrace(
-    begin_it, findFirstGreaterX(begin_it, this->end(), mzEnd), 0);
+    begin_it, end_it, 0);
 }
 
 
@@ -752,6 +796,13 @@ Trace::sortX()
   });
 }
 
+void
+Trace::sortY()
+{
+  std::sort(begin(), end(), [](const DataPoint &a, const DataPoint &b) {
+    return (a.y > b.y);
+  });
+}
 
 void
 Trace::unique()
@@ -765,34 +816,6 @@ Trace::unique()
 }
 
 
-std::vector<pappso_double>
-Trace::xValues()
-{
-  std::vector<pappso_double> values;
-
-  for(auto &&dataPoint : *this)
-    {
-      values.push_back(dataPoint.x);
-    }
-
-  return values;
-}
-
-
-std::vector<pappso_double>
-Trace::yValues()
-{
-  std::vector<pappso_double> values;
-
-  for(auto &&dataPoint : *this)
-    {
-      values.push_back(dataPoint.y);
-    }
-
-  return values;
-}
-
-
 QString
 Trace::toString() const
 {
