@@ -139,6 +139,8 @@ class PMSPP_LIB_DECL Trace : public std::vector<DataPoint>
 
   public:
   Trace();
+  Trace(const std::vector<pappso_double> &xVector,
+        const std::vector<pappso_double> &yVector);
   Trace(const std::vector<std::pair<pappso_double, pappso_double>> &dataPoints);
   Trace(const std::vector<DataPoint> &dataPoints);
   Trace(const std::vector<DataPoint> &&dataPoints);
@@ -160,8 +162,9 @@ class PMSPP_LIB_DECL Trace : public std::vector<DataPoint>
   TraceSPtr makeTraceSPtr() const;
   TraceCstSPtr makeTraceCstSPtr() const;
 
-  std::vector<pappso_double> xToVector() const;
-  std::vector<pappso_double> yToVector() const;
+  std::vector<pappso_double> xValues() const;
+  std::vector<pappso_double> yValues() const;
+
   std::map<pappso_double, pappso_double> toMap() const;
 
   DataPoint containsX(pappso_double value,
@@ -181,12 +184,9 @@ class PMSPP_LIB_DECL Trace : public std::vector<DataPoint>
 
   // was void Spectrum::sortByMz();
   void sortX();
+  void sortY();
   void unique();
 
-
-  std::vector<pappso_double> xValues();
-  std::vector<pappso_double> yValues();
-
   /** @brief apply a filter on this trace
    * @param filter to process the signal
    * @return reference on the modified Trace
@@ -199,9 +199,9 @@ class PMSPP_LIB_DECL Trace : public std::vector<DataPoint>
   //! to \p value.
   // const DataPoint &dataPointWithX(pappso_double value) const;
   std::size_t dataPointIndexWithX(pappso_double value) const;
-  std::vector<DataPoint>::iterator dataPointIteratorxWithX(pappso_double value);
+  std::vector<DataPoint>::iterator dataPointIteratorWithX(pappso_double value);
   std::vector<DataPoint>::const_iterator
-  dataPointCstIteratorxWithX(pappso_double value) const;
+  dataPointCstIteratorWithX(pappso_double value) const;
 };
 
 
