@@ -176,6 +176,23 @@ enum class DataKind : std::int8_t
 };
 
 
+enum class Axis : std::int8_t
+{
+  unset = 0,
+  x = 1,
+  y = 2,
+  z = 3,
+};
+
+
+enum class AxisScale : std::int8_t
+{
+  unset = 0,
+  orig = 1,
+  log10 = 2,
+};
+
+
 /** \def XixExtactMethod method to extract Xic
  *
  */
