@@ -36,6 +36,7 @@
 #include <solvers.h>
 #include <QSqlError>
 #include <QSqlQuery>
+#include <QSqlRecord>
 #include <QMutexLocker>
 #include <QThread>
 #include <set>
@@ -46,7 +47,8 @@ TimsData::TimsData(QDir timsDataDirectory)
   : m_timsDataDirectory(timsDataDirectory)
 {
 
-  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
+  qDebug();
+  mpa_mzCalibrationStore = new MzCalibrationStore();
   if(!m_timsDataDirectory.exists())
     {
       throw PappsoException(
@@ -88,8 +90,7 @@ TimsData::TimsData(QDir timsDataDirectory)
     {
       compression_type = q.value(1).toInt();
     }
-  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__
-           << " compression_type=" << compression_type;
+  qDebug() << " compression_type=" << compression_type;
   mpa_timsBinDec = new TimsBinDec(
     QFileInfo(m_timsDataDirectory.absoluteFilePath("analysis.tdf_bin")),
     compression_type);
@@ -132,16 +133,28 @@ TimsData::TimsData(QDir timsDataDirectory)
       m_totalNumberOfScans = q.value(0).toLongLong();
     }
 
-  /*
-  std::shared_ptr<FilterTriangle> ms2filter =
-    std::make_shared<FilterTriangle>();
-  ms2filter.get()->setTriangleSlope(50, 0.02);
-  mcsp_ms2Filter = ms2filter;
-*/
+  if(!q.exec("select * from MzCalibration;"))
+    {
+      qDebug();
+      throw PappsoException(
+        QObject::tr("ERROR in TIMS sqlite database file %1, executing SQL "
+                    "command %2:\n%3\n%4\n%5")
+          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
+          .arg(q.lastQuery())
+          .arg(q.lastError().databaseText())
+          .arg(q.lastError().driverText())
+          .arg(q.lastError().nativeErrorCode()));
+    }
 
+  while(q.next())
+    {
+      QSqlRecord record = q.record();
+      m_mapMzCalibrationRecord.insert(
+        std::pair<int, QSqlRecord>(record.value(0).toInt(), record));
+    }
 
   std::shared_ptr<pappso::FilterPseudoCentroid> ms2filter =
-    std::make_shared<pappso::FilterPseudoCentroid>(20000, 0.05, 0.5, 0.1);
+    std::make_shared<pappso::FilterPseudoCentroid>(35000, 5, 0.5, 0.1);
   mcsp_ms2Filter = ms2filter;
 
 
@@ -184,7 +197,7 @@ TimsData::openDatabaseConnection() const
 
 TimsData::TimsData([[maybe_unused]] const pappso::TimsData &other)
 {
-  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
+  qDebug();
 }
 
 TimsData::~TimsData()
@@ -196,6 +209,18 @@ TimsData::~TimsData()
     }
 }
 
+void
+TimsData::setMs2BuiltinCentroid(bool centroid)
+{
+  m_builtinMs2Centroid = centroid;
+}
+
+bool
+TimsData::getMs2BuiltinCentroid() const
+{
+  return m_builtinMs2Centroid;
+}
+
 std::pair<std::size_t, std::size_t>
 TimsData::getScanCoordinateFromRawIndex(std::size_t raw_index) const
 {
@@ -307,18 +332,15 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId) const
 
   QSqlDatabase qdb = openDatabaseConnection();
   QSqlQuery q      = qdb.exec(
-    QString("SELECT Frames.TimsId, Frames.AccumulationTime, "
-            " MzCalibration.DigitizerTimebase, MzCalibration.DigitizerDelay, "
-            "MzCalibration.C0, MzCalibration.C1, MzCalibration.C2, "
-            "MzCalibration.C3, MzCalibration.T1, MzCalibration.T2, "
-            "MzCalibration.dC1, MzCalibration.dC2, Frames.T1, Frames.T2, "
-            "Frames.Time, Frames.MsMsType, TimsCalibration.ModelType, "
-            "TimsCalibration.C0, TimsCalibration.C1, TimsCalibration.C2, "
-            "TimsCalibration.C3, TimsCalibration.C4, TimsCalibration.C5, "
-            "TimsCalibration.C6, TimsCalibration.C7, TimsCalibration.C8, "
-            "TimsCalibration.C9   FROM "
-            "Frames INNER JOIN MzCalibration ON "
-            "Frames.MzCalibration=MzCalibration.Id INNER JOIN TimsCalibration "
+    QString("SELECT Frames.TimsId, Frames.AccumulationTime, " // 1
+            "Frames.MzCalibration, "                          // 2
+            "Frames.T1, Frames.T2, "                          // 4
+            "Frames.Time, Frames.MsMsType, TimsCalibration.ModelType, " // 7
+            "TimsCalibration.C0, TimsCalibration.C1, TimsCalibration.C2, " // 10
+            "TimsCalibration.C3, TimsCalibration.C4, TimsCalibration.C5, " // 13
+            "TimsCalibration.C6, TimsCalibration.C7, TimsCalibration.C8, " // 16
+            "TimsCalibration.C9   FROM  Frames" // 17
+            " INNER JOIN TimsCalibration "
             "ON Frames.TimsCalibration=TimsCalibration.Id where "
             "Frames.Id=%1;")
       .arg(timsId));
@@ -341,46 +363,44 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId) const
     {
 
 
-      double T1_ref  = q.value(8).toDouble();
-      double T2_ref  = q.value(9).toDouble();
-      double factor1 = q.value(10).toDouble();
-      double factor2 = q.value(11).toDouble();
-
-      double T1_frame = q.value(12).toDouble();
-      double T2_frame = q.value(13).toDouble();
-
-
-      double temperature_correction =
-        factor1 * (T1_ref - T1_frame) + factor2 * (T2_ref - T2_frame);
-      temperature_correction = (double)1.0 + (temperature_correction / 1.0e6);
-
       tims_frame = std::make_shared<TimsFrameBase>(
         TimsFrameBase(timsId, q.value(0).toUInt()));
 
-      tims_frame.get()->setMzCalibration(temperature_correction,
-                                         q.value(2).toDouble(),
-                                         q.value(3).toDouble(),
-                                         q.value(4).toDouble(),
-                                         q.value(5).toDouble(),
-                                         q.value(6).toDouble(),
-                                         q.value(7).toDouble());
+      auto it_map_record = m_mapMzCalibrationRecord.find(q.value(2).toInt());
+      if(it_map_record != m_mapMzCalibrationRecord.end())
+        {
+
+          double T1_frame = q.value(3).toDouble(); // Frames.T1
+          double T2_frame = q.value(4).toDouble(); // Frames.T2
+
+
+          tims_frame.get()->setMzCalibrationInterfaceSPtr(
+            mpa_mzCalibrationStore->getInstance(
+              T1_frame, T2_frame, it_map_record->second));
+        }
+      else
+        {
+          throw ExceptionNotFound(
+            QObject::tr("ERROR MzCalibration database id %1 not found")
+              .arg(q.value(2).toInt()));
+        }
 
       tims_frame.get()->setAccumulationTime(q.value(1).toDouble());
 
-      tims_frame.get()->setTime(q.value(14).toDouble());
-      tims_frame.get()->setMsMsType(q.value(15).toUInt());
+      tims_frame.get()->setTime(q.value(5).toDouble());
+      tims_frame.get()->setMsMsType(q.value(6).toUInt());
 
-      tims_frame.get()->setTimsCalibration(q.value(16).toInt(),
-                                           q.value(17).toDouble(),
-                                           q.value(18).toDouble(),
-                                           q.value(19).toDouble(),
-                                           q.value(20).toDouble(),
-                                           q.value(21).toDouble(),
-                                           q.value(22).toDouble(),
-                                           q.value(23).toDouble(),
-                                           q.value(24).toDouble(),
-                                           q.value(25).toDouble(),
-                                           q.value(26).toDouble());
+      tims_frame.get()->setTimsCalibration(q.value(7).toInt(),
+                                           q.value(8).toDouble(),
+                                           q.value(9).toDouble(),
+                                           q.value(10).toDouble(),
+                                           q.value(11).toDouble(),
+                                           q.value(12).toDouble(),
+                                           q.value(13).toDouble(),
+                                           q.value(14).toDouble(),
+                                           q.value(15).toDouble(),
+                                           q.value(16).toDouble(),
+                                           q.value(17).toDouble());
       return tims_frame;
     }
 
@@ -430,18 +450,15 @@ TimsData::getTimsFrameCstSPtr(std::size_t timsId) const
 
   QSqlDatabase qdb = openDatabaseConnection();
   QSqlQuery q      = qdb.exec(
-    QString("SELECT Frames.TimsId, Frames.AccumulationTime, "
-            " MzCalibration.DigitizerTimebase, MzCalibration.DigitizerDelay, "
-            "MzCalibration.C0, MzCalibration.C1, MzCalibration.C2, "
-            "MzCalibration.C3, MzCalibration.T1, MzCalibration.T2, "
-            "MzCalibration.dC1, MzCalibration.dC2, Frames.T1, Frames.T2, "
-            "Frames.Time, Frames.MsMsType, TimsCalibration.ModelType, "
-            "TimsCalibration.C0, TimsCalibration.C1, TimsCalibration.C2, "
-            "TimsCalibration.C3, TimsCalibration.C4, TimsCalibration.C5, "
-            "TimsCalibration.C6, TimsCalibration.C7, TimsCalibration.C8, "
-            "TimsCalibration.C9   FROM "
-            "Frames INNER JOIN MzCalibration ON "
-            "Frames.MzCalibration=MzCalibration.Id INNER JOIN TimsCalibration "
+    QString("SELECT Frames.TimsId, Frames.AccumulationTime, " // 1
+            "Frames.MzCalibration, "                          // 2
+            "Frames.T1, Frames.T2, "                          // 4
+            "Frames.Time, Frames.MsMsType, TimsCalibration.ModelType, " // 7
+            "TimsCalibration.C0, TimsCalibration.C1, TimsCalibration.C2, " // 10
+            "TimsCalibration.C3, TimsCalibration.C4, TimsCalibration.C5, " // 13
+            "TimsCalibration.C6, TimsCalibration.C7, TimsCalibration.C8, " // 16
+            "TimsCalibration.C9   FROM  Frames" // 17
+            " INNER JOIN TimsCalibration "
             "ON Frames.TimsCalibration=TimsCalibration.Id where "
             "Frames.Id=%1;")
       .arg(timsId));
@@ -464,46 +481,45 @@ TimsData::getTimsFrameCstSPtr(std::size_t timsId) const
     {
 
 
-      double T1_ref  = q.value(8).toDouble();
-      double T2_ref  = q.value(9).toDouble();
-      double factor1 = q.value(10).toDouble();
-      double factor2 = q.value(11).toDouble();
-
-      double T1_frame = q.value(12).toDouble();
-      double T2_frame = q.value(13).toDouble();
-
-
-      double temperature_correction =
-        factor1 * (T1_ref - T1_frame) + factor2 * (T2_ref - T2_frame);
-      temperature_correction = (double)1.0 + (temperature_correction / 1.0e6);
-
       tims_frame =
         mpa_timsBinDec->getTimsFrameSPtrByOffset(timsId, q.value(0).toUInt());
 
-      tims_frame.get()->setMzCalibration(temperature_correction,
-                                         q.value(2).toDouble(),
-                                         q.value(3).toDouble(),
-                                         q.value(4).toDouble(),
-                                         q.value(5).toDouble(),
-                                         q.value(6).toDouble(),
-                                         q.value(7).toDouble());
+
+      auto it_map_record = m_mapMzCalibrationRecord.find(q.value(2).toInt());
+      if(it_map_record != m_mapMzCalibrationRecord.end())
+        {
+
+          double T1_frame = q.value(3).toDouble(); // Frames.T1
+          double T2_frame = q.value(4).toDouble(); // Frames.T2
+
+
+          tims_frame.get()->setMzCalibrationInterfaceSPtr(
+            mpa_mzCalibrationStore->getInstance(
+              T1_frame, T2_frame, it_map_record->second));
+        }
+      else
+        {
+          throw ExceptionNotFound(
+            QObject::tr("ERROR MzCalibration database id %1 not found")
+              .arg(q.value(2).toInt()));
+        }
 
       tims_frame.get()->setAccumulationTime(q.value(1).toDouble());
 
-      tims_frame.get()->setTime(q.value(14).toDouble());
-      tims_frame.get()->setMsMsType(q.value(15).toUInt());
+      tims_frame.get()->setTime(q.value(5).toDouble());
+      tims_frame.get()->setMsMsType(q.value(6).toUInt());
 
-      tims_frame.get()->setTimsCalibration(q.value(16).toInt(),
-                                           q.value(17).toDouble(),
-                                           q.value(18).toDouble(),
-                                           q.value(19).toDouble(),
-                                           q.value(20).toDouble(),
-                                           q.value(21).toDouble(),
-                                           q.value(22).toDouble(),
-                                           q.value(23).toDouble(),
-                                           q.value(24).toDouble(),
-                                           q.value(25).toDouble(),
-                                           q.value(26).toDouble());
+      tims_frame.get()->setTimsCalibration(q.value(7).toInt(),
+                                           q.value(8).toDouble(),
+                                           q.value(9).toDouble(),
+                                           q.value(10).toDouble(),
+                                           q.value(11).toDouble(),
+                                           q.value(12).toDouble(),
+                                           q.value(13).toDouble(),
+                                           q.value(14).toDouble(),
+                                           q.value(15).toDouble(),
+                                           q.value(16).toDouble(),
+                                           q.value(17).toDouble());
       return tims_frame;
     }
 
@@ -533,6 +549,135 @@ TimsData::getTotalNumberOfPrecursors() const
   return m_totalNumberOfPrecursors;
 }
 
+std::vector<std::size_t>
+TimsData::getPrecursorsFromMzRtCharge(int charge,
+                                      double mz_val,
+                                      double rt_sec,
+                                      double k0)
+{
+  std::vector<std::size_t> precursor_ids;
+  std::vector<std::vector<double>> ids;
+
+  QSqlDatabase qdb = openDatabaseConnection();
+  QSqlQuery q      = qdb.exec(
+    QString(
+      "SELECT Frames.time, Precursors.MonoisotopicMz, Precursors.Charge, "
+      "Precursors.Id, Frames.Id, PasefFrameMsMsInfo.ScanNumBegin, "
+      "PasefFrameMsMsInfo.scanNumEnd "
+      "FROM Frames "
+      "INNER JOIN PasefFrameMsMsInfo ON Frames.Id = PasefFrameMsMsInfo.Frame "
+      "INNER JOIN Precursors ON PasefFrameMsMsInfo.Precursor = Precursors.Id "
+      "WHERE Precursors.Charge == %1 "
+      "AND Precursors.MonoisotopicMz > %2 -0.01 "
+      "AND Precursors.MonoisotopicMz < %2 +0.01 "
+      "AND Frames.Time >= %3 "
+      "AND Frames.Time < %3 +1; ")
+      .arg(charge)
+      .arg(mz_val)
+      .arg(rt_sec));
+  if(q.lastError().isValid())
+    {
+
+      throw PappsoException(
+        QObject::tr("ERROR in TIMS sqlite database file %1, database name %2, "
+                    "executing SQL "
+                    "command %3:\n%4\n%5\n%6")
+          .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf"))
+          .arg(qdb.databaseName())
+          .arg(q.lastQuery())
+          .arg(qdb.lastError().databaseText())
+          .arg(qdb.lastError().driverText())
+          .arg(qdb.lastError().nativeErrorCode()));
+    }
+  while(q.next())
+    {
+      qInfo() << q.value(0).toDouble() << q.value(1).toDouble()
+              << q.value(2).toDouble() << q.value(3).toDouble();
+
+      std::vector<double> sql_values;
+      sql_values.push_back(q.value(4).toDouble()); // frame id
+      sql_values.push_back(q.value(3).toDouble()); // precursor id
+      sql_values.push_back(q.value(5).toDouble()); // scan num begin
+      sql_values.push_back(q.value(6).toDouble()); // scan num end
+      sql_values.push_back(q.value(1).toDouble()); // mz_value
+
+      ids.push_back(sql_values);
+
+
+      if(std::find(precursor_ids.begin(),
+                   precursor_ids.end(),
+                   q.value(3).toDouble()) == precursor_ids.end())
+        {
+          precursor_ids.push_back(q.value(3).toDouble());
+        }
+    }
+
+  if(precursor_ids.size() > 1)
+    {
+      // std::vector<std::size_t> precursor_ids_ko =
+      // getMatchPrecursorIdByKo(ids, values[3]);
+      if(precursor_ids.size() > 1)
+        {
+          precursor_ids = getClosestPrecursorIdByMz(ids, k0);
+        }
+      return precursor_ids;
+    }
+  else
+    {
+      return precursor_ids;
+    }
+}
+
+std::vector<std::size_t>
+TimsData::getMatchPrecursorIdByKo(std::vector<std::vector<double>> ids,
+                                  double ko_value)
+{
+  std::vector<std::size_t> precursor_id;
+  for(std::vector<double> index : ids)
+    {
+      auto coordinate = getScanCoordinateFromRawIndex(index[0]);
+
+      TimsFrameBaseCstSPtr tims_frame;
+      tims_frame = getTimsFrameBaseCstSPtrCached(coordinate.first);
+
+      double bko = tims_frame.get()->getOneOverK0Transformation(index[2]);
+      double eko = tims_frame.get()->getOneOverK0Transformation(index[3]);
+
+      qInfo() << "diff" << (bko + eko) / 2;
+      double mean_ko = (bko + eko) / 2;
+
+      if(mean_ko > ko_value - 0.1 && mean_ko < ko_value + 0.1)
+        {
+          precursor_id.push_back(index[1]);
+        }
+    }
+  return precursor_id;
+}
+
+std::vector<std::size_t>
+TimsData::getClosestPrecursorIdByMz(std::vector<std::vector<double>> ids,
+                                    double mz_value)
+{
+  std::vector<std::size_t> best_precursor;
+  double best_value     = 1;
+  int count             = 1;
+  int best_val_position = 0;
+
+  for(std::vector<double> values : ids)
+    {
+      double new_val = abs(mz_value - values[4]);
+      if(new_val < best_value)
+        {
+          best_value        = new_val;
+          best_val_position = count;
+        }
+      count++;
+    }
+  best_precursor.push_back(ids[best_val_position][1]);
+  return best_precursor;
+}
+
+
 unsigned int
 TimsData::getMsLevelBySpectrumIndex(std::size_t spectrum_index)
 {
@@ -570,10 +715,6 @@ TimsData::getQualifiedMassSpectrumByRawIndex(std::size_t spectrum_index,
   mass_spectrum.setMsLevel(tims_frame.get()->getMsLevel());
   mass_spectrum.setRtInSeconds(tims_frame.get()->getTime());
 
-  mass_spectrum.setPrecursorCharge(0);
-  mass_spectrum.setPrecursorMz(0);
-  mass_spectrum.setPrecursorIntensity(0);
-
   mass_spectrum.setDtInMilliSeconds(
     tims_frame.get()->getDriftTime(coordinate.second));
   // 1/K0
@@ -624,11 +765,16 @@ TimsData::getQualifiedMassSpectrumByRawIndex(std::size_t spectrum_index,
         }
       if(q.next())
         {
-          mass_spectrum.setPrecursorCharge(q.value(11).toInt());
-          mass_spectrum.setPrecursorMz(q.value(10).toDouble());
-          mass_spectrum.setPrecursorIntensity(q.value(13).toDouble());
+          // mass_spectrum.setPrecursorCharge(q.value(11).toInt());
+          // mass_spectrum.setPrecursorMz(q.value(10).toDouble());
+          // mass_spectrum.setPrecursorIntensity(q.value(13).toDouble());
           // mass_spectrum.setPrecursorSpectrumIndex();
 
+          mass_spectrum.appendPrecursorIonData(
+            PrecursorIonData(q.value(10).toDouble(),
+                             q.value(11).toInt(),
+                             q.value(13).toDouble()));
+
 
           MassSpectrumId spectrum_id;
           std::size_t prec_spectrum_index = getRawIndexFromCoordinate(
@@ -671,9 +817,6 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(std::size_t ms2_index,
     {
       QSqlDatabase qdb = openDatabaseConnection();
       mass_spectrum.setMsLevel(1);
-      mass_spectrum.setPrecursorCharge(0);
-      mass_spectrum.setPrecursorMz(0);
-      mass_spectrum.setPrecursorIntensity(0);
       mass_spectrum.setPrecursorSpectrumIndex(0);
       mass_spectrum.setEmptyMassSpectrum(true);
       QSqlQuery q =
@@ -815,10 +958,14 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(std::size_t ms2_index,
             }
         }
     }
+    
+  catch(PappsoException &error)
+    {
+      throw error;
+    }
   catch(std::exception &error)
     {
-      qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-               << QString("Failure %1 ").arg(error.what());
+      qDebug() << QString("Failure %1 ").arg(error.what());
     }
   return mass_spectrum;
 }
@@ -844,9 +991,6 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
       mass_spectrum.setMassSpectrumId(spectrum_id);
 
       mass_spectrum.setMsLevel(2);
-      mass_spectrum.setPrecursorCharge(0);
-      mass_spectrum.setPrecursorMz(0);
-      mass_spectrum.setPrecursorIntensity(0);
       mass_spectrum.setPrecursorSpectrumIndex(ms2_index - 1);
 
       mass_spectrum.setEmptyMassSpectrum(true);
@@ -906,9 +1050,14 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
               tims_frame_list.push_back(q.value(0).toLongLong());
               if(first)
                 {
-                  mass_spectrum.setPrecursorCharge(q.value(11).toInt());
-                  mass_spectrum.setPrecursorMz(q.value(10).toDouble());
-                  mass_spectrum.setPrecursorIntensity(q.value(13).toDouble());
+                  // mass_spectrum.setPrecursorCharge(q.value(11).toInt());
+                  // mass_spectrum.setPrecursorMz(q.value(10).toDouble());
+                  // mass_spectrum.setPrecursorIntensity(q.value(13).toDouble());
+
+                  mass_spectrum.appendPrecursorIonData(
+                    PrecursorIonData(q.value(10).toDouble(),
+                                     q.value(11).toInt(),
+                                     q.value(13).toDouble()));
 
                   mass_spectrum.setPrecursorNativeId(
                     QString("frame=%1 begin=%2 end=%3 precursor=%4 idxms2=%5")
@@ -941,9 +1090,10 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
             }
           // QMutexLocker locker(&m_mutex_spectrum);
           qDebug();
-          pappso::TimsFrameCstSPtr tims_frame;
-          TracePlusCombiner combiner;
-          MapTrace combiner_result;
+          pappso::TimsFrameCstSPtr tims_frame, previous_frame;
+          // TracePlusCombiner combiner;
+          // MapTrace combiner_result;
+          std::map<quint32, quint32> raw_spectrum;
           first = true;
           for(std::size_t tims_id : tims_frame_list)
             {
@@ -970,11 +1120,32 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
               if(want_binary_data)
                 {
                   qDebug();
-                  combiner.combine(combiner_result,
+                  /*combiner.combine(combiner_result,
                                    tims_frame.get()->cumulateScanToTrace(
-                                     scan_mobility_start, scan_mobility_end));
+                                     scan_mobility_start, scan_mobility_end));*/
+                  if(previous_frame.get() != nullptr)
+                    {
+                      if(previous_frame.get()->hasSameCalibrationData(
+                           *tims_frame.get()))
+                        {
+                        }
+                      else
+                        {
+                          throw ExceptionNotFound(
+                            QObject::tr(
+                              "ERROR in %1 %2, different calibration data "
+                              "between frame id %3 and frame id %4")
+                              .arg(__FILE__)
+                              .arg(__FUNCTION__)
+                              .arg(previous_frame.get()->getId())
+                              .arg(tims_frame.get()->getId()));
+                        }
+                    }
+                  tims_frame.get()->cumulateScansInRawMap(
+                    raw_spectrum, scan_mobility_start, scan_mobility_end);
                   qDebug();
                 }
+              previous_frame = tims_frame;
             }
           qDebug() << " precursor_index=" << precursor_index
                    << " num_rows=" << tims_frame_list.size()
@@ -992,9 +1163,20 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
             {
               qDebug();
               // peak_pick.filter(trace);
+              pappso::Trace trace;
+              if(m_builtinMs2Centroid)
+                {
+                  trace =
+                    tims_frame.get()->getTraceFromCumulatedScansBuiltinCentroid(
+                      raw_spectrum);
+                }
+              else
+                {
+                  // no builtin centroid:
 
-              pappso::Trace trace(combiner_result);
-              qDebug() << "AAAAAAA";
+                  trace =
+                    tims_frame.get()->getTraceFromCumulatedScans(raw_spectrum);
+                }
 
               if(trace.size() > 0)
                 {
@@ -1029,10 +1211,14 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
           qDebug();
         }
     }
+
+  catch(PappsoException &error)
+    {
+      throw error;
+    }
   catch(std::exception &error)
     {
-      qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-               << QString("Failure %1 ").arg(error.what());
+      qDebug() << QString("Failure %1 ").arg(error.what());
     }
   qDebug();
 }
@@ -1153,6 +1339,10 @@ TimsData::extractXicListByPrecursorIds(
       extractor_p->setXicExtractMethod(xicExtractMethod);
       extractor_p->extractTimsXicList(xic_list, rtRange);
     }
+  catch(PappsoException &error)
+    {
+      throw error;
+    }
   catch(std::exception &error)
     {
       qDebug() << QString("Failure %1 ").arg(error.what());
