@@ -36,6 +36,7 @@
 #include <deque>
 #include <QMutex>
 #include <QSqlQuery>
+#include "mzcalibration/mzcalibrationstore.h"
 
 namespace pappso
 {
@@ -57,6 +58,11 @@ struct TimsXicStructure
   unsigned int charge;
 };
 
+class TimsData;
+
+/** \brief shared pointer on a TimsData object
+ */
+typedef std::shared_ptr<TimsData> TimsDataSp;
 
 /**
  * @todo write docs
@@ -102,6 +108,14 @@ class PMSPP_LIB_DECL TimsData
    */
   std::size_t getTotalNumberOfPrecursors() const;
 
+  /** @brief guess possible precursor ids given a charge, m/z, retention time
+   * and k0
+   * @return a list of possible precursor ids
+   */
+  std::vector<std::size_t> getPrecursorsFromMzRtCharge(int charge,
+                                                       double mz_val,
+                                                       double rt_sec,
+                                                       double k0);
 
   unsigned int getMsLevelBySpectrumIndex(std::size_t spectrum_index);
 
@@ -118,9 +132,26 @@ class PMSPP_LIB_DECL TimsData
   QualifiedMassSpectrum getQualifiedMs1MassSpectrumByPrecursorId(
     std::size_t ms2_index, std::size_t precursor_index, bool want_binary_data);
 
+  /** @brief filter interface to apply just after raw MS2 specturm extraction
+   * the filter can be a list of filters inside a FilterSuite object
+   */
   void setMs2FilterCstSPtr(pappso::FilterInterfaceCstSPtr &filter);
+
+  /** @brief filter interface to apply just after raw MS1 specturm extraction
+   * the filter can be a list of filters inside a FilterSuite object
+   */
   void setMs1FilterCstSPtr(pappso::FilterInterfaceCstSPtr &filter);
 
+  /** @brief enable or disable simple centroid filter on raw tims data for MS2
+   */
+  void setMs2BuiltinCentroid(bool centroid);
+
+
+  /** @brief tells if simple centroid filter on raw tims data for MS2 is enabled
+   * or not
+   */
+  bool getMs2BuiltinCentroid() const;
+
 
   std::vector<std::size_t> getTimsMS1FrameIdRange(double rt_begin,
                                                   double rt_end) const;
@@ -169,6 +200,16 @@ class PMSPP_LIB_DECL TimsData
     XicExtractMethod xicExtractMethod,
     double rtRange) const;
 
+  std::vector<std::size_t>
+  getMatchPrecursorIdByKo(std::vector<std::vector<double>> ids,
+                          double ko_value);
+
+  /** @todo documentation
+   */
+  std::vector<std::size_t>
+  getClosestPrecursorIdByMz(std::vector<std::vector<double>> ids,
+                            double mz_value);
+
   private:
   QDir m_timsDataDirectory;
   TimsBinDec *mpa_timsBinDec = nullptr;
@@ -181,6 +222,16 @@ class PMSPP_LIB_DECL TimsData
 
   pappso::FilterInterfaceCstSPtr mcsp_ms2Filter = nullptr;
   pappso::FilterInterfaceCstSPtr mcsp_ms1Filter = nullptr;
+
+  /** @brief enable builtin centroid on raw tims integers by default
+   */
+  bool m_builtinMs2Centroid = true;
+
+
+  std::map<int, QSqlRecord> m_mapMzCalibrationRecord;
+
+  MzCalibrationStore * mpa_mzCalibrationStore;
+
   QMutex m_mutex;
 };
 } // namespace pappso
