@@ -45,10 +45,13 @@ TimsFrame::XicComputeStructure::XicComputeStructure(
 
   mobilityIndexBegin = xic_struct.scanNumBegin;
   mobilityIndexEnd   = xic_struct.scanNumEnd;
-  mzIndexLowerBound  = fram_p->getRawIndexFromMz(
-    xic_struct.mzRange.lower()); // convert mz to raw digitizer value
-  mzIndexUpperBound = fram_p->getRawIndexFromMz(xic_struct.mzRange.upper());
-  tmpIntensity      = 0;
+  mzIndexLowerBound =
+    fram_p->getMzCalibrationInterfaceSPtr().get()->getTofIndexFromMz(
+      xic_struct.mzRange.lower()); // convert mz to raw digitizer value
+  mzIndexUpperBound =
+    fram_p->getMzCalibrationInterfaceSPtr().get()->getTofIndexFromMz(
+      xic_struct.mzRange.upper());
+  tmpIntensity = 0;
 }
 
 TimsFrame::TimsFrame(std::size_t timsId,
@@ -217,7 +220,7 @@ TimsFrame::cumulateScan(std::size_t scanNum,
 
   if(m_timsDataFrame.size() == 0)
     return;
-  checkScanNum(scanNum);
+  // checkScanNum(scanNum);
 
 
   std::size_t size = getNbrPeaks(scanNum);
@@ -274,10 +277,15 @@ TimsFrame::cumulateScanToTrace(std::size_t scanNumBegin,
       qDebug();
       pappso::DataPoint data_point_cumul;
 
+
+      MzCalibrationInterface *mz_calibration_p =
+        getMzCalibrationInterfaceSPtr().get();
+
+
       for(std::pair<quint32, quint32> pair_tof_intensity : raw_spectrum)
         {
           data_point_cumul.x =
-            getMzFromTof(getTofFromIndex(pair_tof_intensity.first));
+            mz_calibration_p->getMzFromTofIndex(pair_tof_intensity.first);
           // normalization
           data_point_cumul.y =
             pair_tof_intensity.second * ((double)100.0 / m_accumulationTime);
@@ -289,8 +297,7 @@ TimsFrame::cumulateScanToTrace(std::size_t scanNumBegin,
 
   catch(std::exception &error)
     {
-      qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()"
-               << QString(
+      qDebug() << QString(
                     "Failure in TimsFrame::cumulateScanToTrace %1 to %2 :\n %3")
                     .arg(scanNumBegin, scanNumEnd)
                     .arg(error.what());
@@ -299,124 +306,40 @@ TimsFrame::cumulateScanToTrace(std::size_t scanNumBegin,
 }
 
 
-pappso::Trace
-TimsFrame::getTraceFromCumulatedScans(
-  std::map<quint32, quint32> &accumulated_scans) const
+void
+TimsFrame::cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum,
+                                 std::size_t scanNumBegin,
+                                 std::size_t scanNumEnd) const
 {
   qDebug();
-  // qDebug();
-  // add flanking peaks
-  std::vector<quint32> keys;
-  transform(begin(accumulated_scans),
-            end(accumulated_scans),
-            back_inserter(keys),
-            [](std::map<quint32, quint32>::value_type const &pair) {
-              return pair.first;
-            });
-  std::sort(keys.begin(), keys.end());
-  pappso::DataPoint data_point_cumul;
-  data_point_cumul.x = 0;
-  quint32 previous   = 0;
-  quint32 hole_width = 2;
 
-  pappso::Trace local_trace;
-
-  bool go_up                = true;
-  double previous_intensity = 0;
-  /*
-  for(quint32 key : keys)
-  {
-
-              data_point_cumul.x =
-                getMzFromTof(getTofFromIndex(key));
-              // normalization
-              data_point_cumul.y =
-                accumulated_scans[key] * ((double)100.0 / m_accumulationTime);
-              mass_spectrum_sptr.get()->push_back(data_point_cumul);
-    }
-*/
-  for(quint32 key : keys)
+  if(m_timsDataFrame.size() == 0)
+    return;
+  try
     {
-      if((key - previous) > hole_width)
+
+      std::size_t imax = scanNumEnd + 1;
+      qDebug();
+      for(std::size_t i = scanNumBegin; i < imax; i++)
         {
-
-          // qDebug() << " key:" << key << " previous:" << previous
-          //       << " (key - previous):" << (key - previous);
-          if(data_point_cumul.x != 0)
-            {
-              data_point_cumul.x = data_point_cumul.x / data_point_cumul.y;
-
-              data_point_cumul.x =
-                getMzFromTof(getTofFromIndex(data_point_cumul.x));
-              // normalization
-              data_point_cumul.y =
-                data_point_cumul.y * ((double)100.0 / m_accumulationTime);
-              local_trace.push_back(data_point_cumul);
-            }
-          // reset
-          data_point_cumul.x = 0;
-          data_point_cumul.y = 0;
-          previous_intensity = 0;
-          go_up              = true;
+          qDebug() << i;
+          cumulateScan(i, rawSpectrum);
+          qDebug() << i;
+          // local_accumulationTime += m_accumulationTime;
         }
-      double intensity = accumulated_scans[key];
-
-      if(previous_intensity < intensity)
-        {
-          if(!go_up)
-            {
-              // stop
-              data_point_cumul.x = data_point_cumul.x / data_point_cumul.y;
-
-              data_point_cumul.x =
-                getMzFromTof(getTofFromIndex(data_point_cumul.x));
-              // normalization
-              data_point_cumul.y =
-                data_point_cumul.y * ((double)100.0 / m_accumulationTime);
-              local_trace.push_back(data_point_cumul);
-
-              // reset
-              data_point_cumul.x = 0;
-              data_point_cumul.y = 0;
-              previous_intensity = 0;
-              go_up              = true;
-            }
-        }
-      else
-        {
-          if(go_up)
-            {
-              go_up = false;
-            }
-          else
-            {
-            }
-        }
-
-
-      data_point_cumul.x += ((double)key * intensity);
-      data_point_cumul.y += intensity;
-      previous           = key;
-      previous_intensity = intensity;
     }
 
-
-  if(data_point_cumul.x != 0)
+  catch(std::exception &error)
     {
-      data_point_cumul.x = data_point_cumul.x / data_point_cumul.y;
-
-      data_point_cumul.x = getMzFromTof(getTofFromIndex(data_point_cumul.x));
-      // normalization
-      data_point_cumul.y =
-        data_point_cumul.y * ((double)100.0 / m_accumulationTime);
-      local_trace.push_back(data_point_cumul);
+      qDebug() << QString("Failure in %1 %2 to %3 :\n %4")
+                    .arg(__FUNCTION__)
+                    .arg(scanNumBegin)
+                    .arg(scanNumEnd)
+                    .arg(error.what());
     }
-  local_trace.sortX();
-  qDebug();
-  // qDebug();
-  return local_trace;
 }
 
+
 pappso::MassSpectrumCstSPtr
 TimsFrame::getMassSpectrumCstSPtr(std::size_t scanNum) const
 {
@@ -446,25 +369,30 @@ TimsFrame::getMassSpectrumSPtr(std::size_t scanNum) const
 
   std::size_t offset = getScanOffset(scanNum);
 
+  MzCalibrationInterface *mz_calibration_p =
+    getMzCalibrationInterfaceSPtr().get();
+
+
   qint32 previous = -1;
+  qint32 tof_index;
   std::vector<quint32> index_list;
+  DataPoint data_point;
   for(std::size_t i = 0; i < size; i++)
     {
-      DataPoint data_point(
+      tof_index =
         (*(quint32 *)((m_timsDataFrame.constData() + (offset * 4) + (i * 8))) +
-         previous),
-        (*(quint32 *)(m_timsDataFrame.constData() + (offset * 4) + (i * 8) +
-                      4)));
+         previous);
+      data_point.y = (*(quint32 *)(m_timsDataFrame.constData() + (offset * 4) +
+                                   (i * 8) + 4));
 
       // intensity normalization
       data_point.y *= 100.0 / m_accumulationTime;
 
-      previous = data_point.x;
+      previous = tof_index;
 
 
       // mz calibration
-      double tof   = (data_point.x * m_digitizerTimebase) + m_digitizerDelay;
-      data_point.x = getMzFromTof(tof);
+      data_point.x = mz_calibration_p->getMzFromTofIndex(tof_index);
       mass_spectrum_sptr.get()->push_back(data_point);
     }
   qDebug();
