@@ -84,6 +84,15 @@ class TimsFrame : public TimsFrameBase
   virtual Trace cumulateScanToTrace(std::size_t scanNumBegin,
                                     std::size_t scanNumEnd) const override;
 
+  /** @brief cumulate scan list into a trace into a raw spectrum map
+   * @param rawSpectrum simple map of integers to cumulate raw counts
+   * @param scanNumBegin first scan to cumulate
+   * @param scanNumEnd last scan to cumulate
+   */
+  virtual void cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum,
+                                     std::size_t scanNumBegin,
+                                     std::size_t scanNumEnd) const override;
+
   /** @brief cumulate a scan list into a map of integers
    * @param scanNumBegin first scan to cumulate
    * @param scanNumEnd last scan to cumulate
@@ -111,11 +120,6 @@ class TimsFrame : public TimsFrameBase
   getMassSpectrumSPtr(std::size_t scanNum) const override;
 
 
-  /** @brief transform accumulation of raw scans into a real mass spectrum
-   */
-  pappso::Trace getTraceFromCumulatedScans(
-    std::map<quint32, quint32> &accumulated_scans) const;
-
   protected:
   void extractTimsXicListInRtRange(
     std::vector<TimsXicStructure>::iterator &itXicListbegin,
