@@ -31,6 +31,7 @@
 #include <vector>
 #include <QtGlobal>
 #include "../../massspectrum/massspectrum.h"
+#include "mzcalibration/mzcalibrationinterface.h"
 
 namespace pappso
 {
@@ -63,23 +64,43 @@ class TimsFrameBase
    */
   ~TimsFrameBase();
 
+  /** @brief tells if 2 tims frame has the same calibration data
+   * Usefull to know if raw data can be handled between frames
+   */
+  virtual bool hasSameCalibrationData(const TimsFrameBase &other) const;
 
   virtual std::size_t getNbrPeaks(std::size_t scanNum) const;
   virtual MassSpectrumSPtr getMassSpectrumSPtr(std::size_t scanNum) const;
   virtual Trace cumulateScanToTrace(std::size_t scanNumBegin,
                                     std::size_t scanNumEnd) const;
 
+
+  /** @brief cumulate scan list into a trace into a raw spectrum map
+   * @param rawSpectrum simple map of integers to cumulate raw counts
+   * @param scanNumBegin first scan to cumulate
+   * @param scanNumEnd last scan to cumulate
+   */
+  virtual void cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum,
+                                     std::size_t scanNumBegin,
+                                     std::size_t scanNumEnd) const;
+
   bool checkScanNum(std::size_t scanNum) const;
 
 
   void setAccumulationTime(double accumulation_time_ms);
-  void setMzCalibration(double temperature_correction,
+  void setMzCalibration(double T1_frame,
+                        double T2_frame,
                         double digitizerTimebase,
                         double digitizerDelay,
                         double C0,
                         double C1,
                         double C2,
-                        double C3);
+                        double C3,
+                        double C4,
+                        double T1_ref,
+                        double T2_ref,
+                        double dC1,
+                        double dC2);
   void setTimsCalibration(int tims_model_type,
                           double C0,
                           double C1,
@@ -110,18 +131,6 @@ class TimsFrameBase
   double getOneOverK0Transformation(std::size_t scanNum) const;
 
 
-  /** @brief get m/z from time of flight
-   * @param tof time of flight
-   * @return m/z value
-   */
-  double getMzFromTof(double tof) const;
-
-  /** @brief get raw index of a given m/z
-   * @param mz the mass to transform
-   * @return integer x raw value
-   */
-  quint32 getRawIndexFromMz(double mz) const;
-
   /** @brief get the scan number from a given 1/Ko mobility value
    * @param one_over_k0 the mobility value to tranform
    * @return integer the scan number
@@ -131,16 +140,24 @@ class TimsFrameBase
 
   double getVoltageTransformation(std::size_t scanNum) const;
 
-  /** @brief get time of flight from raw index
-   * @param index digitizer x raw value
-   * @return tof time of flight
-   */
-  double getTofFromIndex(quint32 index) const;
 
-  /** @brief get time of flight from double index
+  /** @brief transform accumulation of raw scans into a real mass spectrum
    */
-  double getTofFromIndex(double index) const;
+  pappso::Trace getTraceFromCumulatedScans(
+    std::map<quint32, quint32> &accumulated_scans) const;
 
+  /** @brief transform accumulation of raw scans into a real mass spectrum with
+   * a simple centroid on raw integers
+   */
+  pappso::Trace getTraceFromCumulatedScansBuiltinCentroid(
+    std::map<quint32, quint32> &accumulated_scans) const;
+
+  /** @brief get the MzCalibration model to compute mz and TOF for this frame
+   */
+  virtual const MzCalibrationInterfaceSPtr &
+  getMzCalibrationInterfaceSPtr() const final;
+
+  void setMzCalibrationInterfaceSPtr(MzCalibrationInterfaceSPtr mzCalibration);
 
   protected:
   /** @brief total number of scans contained in this frame
@@ -157,13 +174,6 @@ class TimsFrameBase
    */
   double m_accumulationTime = 0;
 
-  double m_digitizerTimebase = 0;
-  double m_digitizerDelay    = 0;
-
-  /** @brief MZ calibration parameters
-   */
-  std::vector<double> m_mzCalibrationArr;
-
   quint8 m_msMsType = 0;
 
   /** @brief retention time
@@ -180,5 +190,7 @@ class TimsFrameBase
   double m_timsVmax   = 0; // C9 from TimsCalibration
   double m_timsC6     = 0;
   double m_timsC7     = 0;
+
+  MzCalibrationInterfaceSPtr msp_mzCalibration = nullptr;
 };
 } // namespace pappso
