@@ -206,7 +206,6 @@ MassSpectrumWidget::peptideAnnotate()
         }
       else
         {
-          
         }
       computeIsotopeMassList();
     }
@@ -389,6 +388,7 @@ MassSpectrumWidget::peakChangeEvent(const DataPoint *p_peak_match)
           peak_shp = _p_mouse_peak->makeDataPointCstSPtr();
           emit peakChanged(peak_shp);
           // try to find matched ion (if it exists)
+          bool found = false;
           for(const PeakIonIsotopeMatch &peak_ion_match :
               _peak_ion_isotope_match_list)
             {
@@ -397,8 +397,13 @@ MassSpectrumWidget::peakChangeEvent(const DataPoint *p_peak_match)
                   // found
                   emit ionChanged(std::make_shared<const PeakIonIsotopeMatch>(
                     PeakIonIsotopeMatch(peak_ion_match)));
+                  found = true;
                 }
             }
+          if(!found)
+            {
+              emit ionChanged(std::shared_ptr<const PeakIonIsotopeMatch>());
+            }
         }
       else
         {
