@@ -89,8 +89,37 @@ BaseTracePlotWidget::setGraphData(QCPGraph *graph_p,
   if(graph_p == nullptr)
     qFatal("Pointer cannot be nullptr.");
 
-  graph_p->setData(QVector<double>::fromStdVector(keys),
-                   QVector<double>::fromStdVector(values));
+  // Version that is now deprecated (20200924)
+  // graph_p->setData(QVector<double>::fromStdVector(keys),
+  // QVector<double>::fromStdVector(values));
+
+  QVector<double> key_qvector;
+  QVector<double> value_qvector;
+
+#pragma GCC warning "Filippo Rusconi: Please check if the bug was fixed in Qt"
+
+#if 0
+                  // Now replace the graph's data. Note that the data are
+                  // inherently sorted (true below).
+                  
+                  // The begin() -- end() ranges constructor did not work as of
+                  // Qt 5.14.2 this day: 20200721
+
+                  key_qvector =
+                    QVector(keys.begin(),
+                            keys.end());
+                  value_qvector =
+                    QVector(values.begin(),
+                            values.end());
+#endif
+
+  for(auto &value : keys)
+    key_qvector.push_back(value);
+
+  for(auto &value : values)
+    value_qvector.push_back(value);
+
+  graph_p->setData(key_qvector, value_qvector, true);
 
   graph_p->setPen(m_pen);
 
@@ -130,8 +159,37 @@ BaseTracePlotWidget::addTrace(const pappso::Trace &trace, const QColor &color)
 
   QCPGraph *graph_p = addGraph();
 
-  graph_p->setData(QVector<double>::fromStdVector(trace.xToVector()),
-                   QVector<double>::fromStdVector(trace.yToVector()));
+  // Now depracated as of 20200924
+  //graph_p->setData(QVector<double>::fromStdVector(trace.xValues()),
+                   //QVector<double>::fromStdVector(trace.yValues()));
+
+  QVector<double> key_qvector;
+  QVector<double> value_qvector;
+
+#pragma GCC warning "Filippo Rusconi: Please check if the bug was fixed in Qt"
+
+#if 0
+                  // Now replace the graph's data. Note that the data are
+                  // inherently sorted (true below).
+                  
+                  // The begin() -- end() ranges constructor did not work as of
+                  // Qt 5.14.2 this day: 20200721
+
+                  key_qvector =
+                    QVector(trace.xValues().begin(),
+                            .trace.xValues()end());
+                  value_qvector =
+                    QVector(trace.yValues().begin(),
+                            trace.yValues().end());
+#endif
+
+  for(auto &value : trace.xValues())
+    key_qvector.push_back(value);
+
+  for(auto &value : trace.yValues())
+    value_qvector.push_back(value);
+
+  graph_p->setData(key_qvector, value_qvector, true);
 
   QPen pen = graph()->pen();
   pen.setColor(color);
