@@ -152,7 +152,9 @@ PrecisionWidget::setCurrentIndex(int index)
         }
       else
         {
-					throw ExceptionNotPossible("precisionwidget.cpp @ setCurrentIndex(int index) -- ERROR programming error.");
+          throw ExceptionNotPossible(
+            "precisionwidget.cpp @ setCurrentIndex(int index) -- ERROR "
+            "programming error.");
         }
     }
 }
@@ -217,7 +219,8 @@ PrecisionWidget::getPrecision() const
     }
   else
     {
-			throw ExceptionNotPossible("precisionwidget.cpp @ getPrecision()-- ERROR programming error.");
+      throw ExceptionNotPossible(
+        "precisionwidget.cpp @ getPrecision()-- ERROR programming error.");
     }
 }
 
@@ -263,9 +266,11 @@ PrecisionWidget::setPrecision(PrecisionPtr precision)
       mp_ppmValueSpinBox->setVisible(false);
     }
   else
-	{
-			throw ExceptionNotPossible("precisionwidget.cpp @ setPrecision(PrecisionPtr precision)-- ERROR programming error.");
-	}
+    {
+      throw ExceptionNotPossible(
+        "precisionwidget.cpp @ setPrecision(PrecisionPtr precision)-- ERROR "
+        "programming error.");
+    }
 }
 
 
